@@ -17,6 +17,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "scim_anthy_factory.h"
+#include "scim_anthy_imengine.h"
 #include "scim_anthy_conversion.h"
 #include "scim_anthy_utils.h"
 
@@ -68,8 +70,9 @@ ConversionSegment::set_reading_length (unsigned int len)
 }
 
 
-Conversion::Conversion (Reading &reading)
-    : m_reading          (reading),
+Conversion::Conversion (AnthyInstance &anthy, Reading &reading)
+    : m_anthy            (anthy),
+      m_reading          (reading),
       m_anthy_context    (anthy_create_context()),
       m_start_id         (0),
       m_cur_segment      (-1)
@@ -257,9 +260,9 @@ Conversion::get_reading_substr (WideString &string,
     case SCIM_ANTHY_CANDIDATE_WIDE_LATIN:
         if (prev_cand == SCIM_ANTHY_CANDIDATE_WIDE_LATIN) {
             String str;
-            convert_to_half (str, m_segments[segment_id].get_string ());
+            util_convert_to_half (str, m_segments[segment_id].get_string ());
             rotate_case (str);
-            convert_to_wide (string, str);
+            util_convert_to_wide (string, str);
         } else {
             m_reading.get (string, seg_start, seg_len,
                            SCIM_ANTHY_STRING_WIDE_LATIN);
@@ -389,16 +392,49 @@ Conversion::get_attribute_list (void)
          it++, seg_id++)
     {
         // create attribute for this segment
-        Attribute attr (pos, it->get_string().length (),
-                        SCIM_ATTR_DECORATE);
-        if ((int) seg_id == m_cur_segment)
-            attr.set_value (SCIM_ATTR_DECORATE_REVERSE);
-        else
-            attr.set_value (SCIM_ATTR_DECORATE_UNDERLINE);
+        if (it->get_string().length () <= 0) {
+            pos += it->get_string().length ();
+            continue;
+        }
 
-        // join
-        if (it->get_string().length () > 0)
-            attrs.push_back (attr);
+        if ((int) seg_id == m_cur_segment) {
+            util_create_attributes (
+                attrs, pos, it->get_string().length(),
+                m_anthy.get_factory()->m_selected_segment_style,
+                m_anthy.get_factory()->m_selected_segment_fg_color,
+                m_anthy.get_factory()->m_selected_segment_bg_color);
+#if 0
+            attrs.push_back (
+                Attribute (pos, it->get_string().length(),
+                           SCIM_ATTR_FOREGROUND,
+                           m_anthy.get_factory()->m_selected_segment_fg_color));
+            attrs.push_back (
+                Attribute (pos, it->get_string().length(),
+                           SCIM_ATTR_BACKGROUND,
+                           m_anthy.get_factory()->m_selected_segment_bg_color));
+#endif
+        } else {
+            util_create_attributes (
+                attrs, pos, it->get_string().length(),
+                m_anthy.get_factory()->m_conversion_style,
+                m_anthy.get_factory()->m_conversion_fg_color,
+                m_anthy.get_factory()->m_conversion_bg_color);
+#if 0
+            attrs.push_back (
+                Attribute (pos, it->get_string().length(),
+                           SCIM_ATTR_FOREGROUND,
+                           m_anthy.get_factory()->m_conversion_fg_color));
+            attrs.push_back (
+                Attribute (pos, it->get_string().length(),
+                           SCIM_ATTR_BACKGROUND,
+                           m_anthy.get_factory()->m_conversion_bg_color));
+#endif
+#if 0
+            attrs.push_back (Attribute (pos, it->get_string().length(),
+                                        SCIM_ATTR_DECORATE,
+                                        SCIM_ATTR_DECORATE_UNDERLINE));
+#endif
+        }
 
         pos += it->get_string().length ();
     }
@@ -574,6 +610,8 @@ Conversion::get_candidates (CommonLookupTable &table, int segment_id)
 
         free (buf);
     }
+
+    table.set_cursor_pos (get_selected_candidate ());
 }
 
 int
