@@ -1,8 +1,28 @@
-/* FIXME! this table should be separated as external file */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/*
+ *  Copyright (C) 2004-2005 Takuro Ashie
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
+#define Uses_SCIM_EVENT
 #include <scim.h>
 #include "scim_anthy_default_tables.h"
 
+using namespace scim;
+
 /* from Suikyo */
 ConvRule scim_anthy_romaji_typing_rule[] = {
 {"-",	"ー",	""},
@@ -235,7 +255,7 @@ ConvRule scim_anthy_romaji_typing_rule[] = {
 {NULL,	NULL,	NULL},
 };
 
-ConvRule scim_anthy_romaji_consonant_rule[] = {
+ConvRule scim_anthy_romaji_double_consonant_rule[] = {
 {"vv",	"っ",	"v"},
 {"xx",	"っ",	"x"},
 {"kk",	"っ",	"k"},
@@ -257,7 +277,7 @@ ConvRule scim_anthy_romaji_consonant_rule[] = {
 {NULL,  NULL,   NULL},
 };
 
-ConvRule scim_anthy_romaji_half_symbol_rule[] = {
+ConvRule scim_anthy_half_symbol_rule[] = {
 {",",	",",	""},
 {".",	".",	""},
 {"!",	"!",	""},
@@ -270,6 +290,7 @@ ConvRule scim_anthy_romaji_half_symbol_rule[] = {
 {"(",	"(",	""},
 {")",	")",	""},
 {"~",	"~",	""},
+{"-",	"-",	""},
 {"=",	"=",	""},
 {"^",	"^",	""},
 {"\\",	"\\",	""},
@@ -292,7 +313,7 @@ ConvRule scim_anthy_romaji_half_symbol_rule[] = {
 {NULL,	NULL,	NULL},
 };
 
-ConvRule scim_anthy_romaji_wide_symbol_rule[] = {
+ConvRule scim_anthy_wide_symbol_rule[] = {
 {",",	"、",	""},
 {".",	"。",	""},
 {"!",	"！",	""},
@@ -305,6 +326,7 @@ ConvRule scim_anthy_romaji_wide_symbol_rule[] = {
 {"(",	"（",	""},
 {")",	"）",	""},
 {"~",	"〜",	""},
+{"-",	"ー",	""},
 {"=",	"＝",	""},
 {"^",	"＾",	""},
 {"\\",	"＼",	""},
@@ -327,7 +349,7 @@ ConvRule scim_anthy_romaji_wide_symbol_rule[] = {
 {NULL,	NULL,	NULL},
 };
 
-ConvRule scim_anthy_romaji_half_number_rule[] = {
+ConvRule scim_anthy_half_number_rule[] = {
 {"1", "1", ""},
 {"2", "2", ""},
 {"3", "3", ""},
@@ -342,7 +364,7 @@ ConvRule scim_anthy_romaji_half_number_rule[] = {
 };
 
 
-ConvRule scim_anthy_romaji_wide_number_rule[] = {
+ConvRule scim_anthy_wide_number_rule[] = {
 {"1", "１", ""},
 {"2", "２", ""},
 {"3", "３", ""},
@@ -357,12 +379,62 @@ ConvRule scim_anthy_romaji_wide_number_rule[] = {
 };
 
 ConvRule scim_anthy_kana_typing_rule[] = {
-#if 0
-{">",	"。",	""},
-{"<",	"、",	""},
-#endif
+// no modifiers keys
+{"1",	"ぬ",	""},
+{"2",	"",	"ふ"},
+{"3",	"あ",	""},
+{"4",	"う",	""},
+{"5",	"え",	""},
+{"6",	"お",	""},
+{"7",	"や",	""},
+{"8",	"ゆ",	""},
+{"9",	"よ",	""},
+{"0",	"わ",	""},
+{"-",	"",	"ほ"},
+{"^",	"",	"へ"},
+
+{"q",	"",	"た"},
+{"w",	"",	"て"},
+{"e",	"い",	""},
+{"r",	"",	"す"},
+{"t",	"",	"か"},
+{"y",	"ん",	""},
+{"u",	"な",	""},
+{"i",	"に",	""},
+{"o",	"ら",	""},
+{"p",	"",	"せ"},
+{"@",	"゛",	""},
+{"[",	"゜",	""},
+
+{"a",	"",	"ち"},
+{"s",	"",	"と"},
+{"d",	"",	"し"},
+{"f",	"",	"は"},
+{"g",	"",	"き"},
+{"h",	"",	"く"},
+{"j",	"ま",	""},
+{"k",	"の",	""},
+{"l",	"り",	""},
+{";",	"れ",	""},
+{":",	"",	"け"},
+{"]",	"む",	""},
+
+{"z",	"",	"つ"},
+{"x",	"",	"さ"},
+{"c",	"",	"そ"},
+{"v",	"",	"ひ"},
+{"b",	"",	"こ"},
+{"n",	"み",	""},
+{"m",	"も",	""},
+{",",	"ね",	""},
+{".",	"る",	""},
+{"/",	"め",	""},
+{"\\",	"ろ",	""},
+
+// shift modifiered keys
+{"!",	"ぬ",	""},
+{"\"",	"",	"ふ"},
 {"#",	"ぁ",	""},
-{"E",	"ぃ",	""},
 {"$",	"ぅ",	""},
 {"%",	"ぇ",	""},
 {"&",	"ぉ",	""},
@@ -370,53 +442,48 @@ ConvRule scim_anthy_kana_typing_rule[] = {
 {"(",	"ゅ",	""},
 {")",	"ょ",	""},
 {"~",	"を",	""},
-{"Z",	"っ",	""},
-{"y",	"ん",	""},
-{"3",	"あ",	""},
-{"e",	"い",	""},
-{"4",	"う",	""},
-{"5",	"え",	""},
-{"6",	"お",	""},
-{"t",	"",	"か"},
-{"g",	"",	"き"},
-{"h",	"",	"く"},
-{":",	"",	"け"},
-{"b",	"",	"こ"},
-{"x",	"",	"さ"},
-{"d",	"",	"し"},
-{"r",	"",	"す"},
-{"p",	"",	"せ"},
-{"c",	"",	"そ"},
-{"q",	"",	"た"},
-{"a",	"",	"ち"},
-{"z",	"",	"つ"},
-{"w",	"",	"て"},
-{"s",	"",	"と"},
-{"u",	"な",	""},
-{"i",	"に",	""},
-{"1",	"ぬ",	""},
-{",",	"ね",	""},
-{"k",	"の",	""},
-{"f",	"",	"は"},
-{"v",	"",	"ひ"},
-{"2",	"",	"ふ"},
-{"^",	"",	"へ"},
-{"-",	"",	"ほ"},
-{"j",	"ま",	""},
-{"n",	"み",	""},
-{"]",	"む",	""},
-{"/",	"め",	""},
-{"m",	"も",	""},
-{"7",	"や",	""},
-{"8",	"ゆ",	""},
-{"9",	"よ",	""},
-{"o",	"ら",	""},
-{"l",	"り",	""},
-{".",	"る",	""},
-{";",	"れ",	""},
-{"0",	"わ",	""},
+{"=",	"",	"ほ"},
 {"|",	"ー",	""},
-{"\\",	"ろ",	""},
+
+{"Q",	"",	"た"},
+{"W",	"",	"て"},
+{"E",	"ぃ",	""},
+{"R",	"",	"す"},
+{"T",	"ヵ",	""},
+{"Y",	"ん",	""},
+{"U",	"な",	""},
+{"I",	"に",	""},
+{"O",	"ら",	""},
+{"P",	"",	"せ"},
+{"`",	"゛",	""},
+{"{",	"「",	""},
+
+{"A",	"",	"ち"},
+{"S",	"",	"と"},
+{"D",	"",	"し"},
+{"F",	"ゎ",	""},
+{"G",	"",	"き"},
+{"H",	"",	"く"},
+{"J",	"ま",	""},
+{"K",	"の",	""},
+{"L",	"り",	""},
+{"+",	"れ",	""},
+{"*",	"ヶ",	""},
+{"}",	"」",	""},
+
+{"Z",	"っ",	""},
+{"X",	"",	"さ"},
+{"C",	"",	"そ"},
+{"V",	"",	"ゐ"},
+{"B",	"",	"こ"},
+{"M",	"も",	""},
+{"N",	"み",	""},
+{"?",	"・",	""},
+{"_",	"ろ",	""},
+{NULL,	NULL,	NULL},
+};
+
+ConvRule scim_anthy_kana_voiced_consonant_rule[] = {
 {"か@",	"が",	""},
 {"き@",	"ぎ",	""},
 {"く@",	"ぐ",	""},
@@ -437,17 +504,31 @@ ConvRule scim_anthy_kana_typing_rule[] = {
 {"ふ@",	"ぶ",	""},
 {"へ@",	"べ",	""},
 {"ほ@",	"ぼ",	""},
+{"か`",	"が",	""},
+{"き`",	"ぎ",	""},
+{"く`",	"ぐ",	""},
+{"け`",	"げ",	""},
+{"こ`",	"ご",	""},
+{"さ`",	"ざ",	""},
+{"し`",	"じ",	""},
+{"す`",	"ず",	""},
+{"せ`",	"ぜ",	""},
+{"そ`",	"ぞ",	""},
+{"た`",	"だ",	""},
+{"ち`",	"ぢ",	""},
+{"つ`",	"づ",	""},
+{"て`",	"で",	""},
+{"と`",	"ど",	""},
+{"は`",	"ば",	""},
+{"ひ`",	"び",	""},
+{"ふ`",	"ぶ",	""},
+{"へ`",	"べ",	""},
+{"ほ`",	"ぼ",	""},
 {"は[",	"ぱ",	""},
 {"ひ[",	"ぴ",	""},
 {"ふ[",	"ぷ",	""},
 {"へ[",	"ぺ",	""},
 {"ほ[",	"ぽ",	""},
-{"?",	"・",	""},
-{"@",	"゛",	""},
-{"[",	"゜",	""},
-{"{",	"「",	""},
-{"}",	"」",	""},
-{NULL,	NULL,	NULL},
 };
 
 ConvRule scim_anthy_romaji_ja_period_rule[] = {
@@ -645,7 +726,6 @@ HiraganaKatakanaRule scim_anthy_hiragana_katakana_table[] = {
 {NULL,	NULL,	NULL},
 };
 
-/* from uim */
 WideRule scim_anthy_wide_table[] = {
 {"a", "ａ"},
 {"b", "ｂ"},
@@ -739,6 +819,207 @@ WideRule scim_anthy_wide_table[] = {
 {">", "＞"},
 {"?", "？"},
 {"/", "／"},
-{"_",  "＿"},
+{"_", "＿"},
+{"*", "＊"},
 {NULL, NULL},
 };
+
+VoicedConsonantRule scim_anthy_voiced_consonant_table[] = {
+{"か",	"が",	""},
+{"き",	"ぎ",	""},
+{"く",	"ぐ",	""},
+{"け",	"げ",	""},
+{"こ",	"ご",	""},
+{"さ",	"ざ",	""},
+{"し",	"じ",	""},
+{"す",	"ず",	""},
+{"せ",	"ぜ",	""},
+{"そ",	"ぞ",	""},
+{"た",	"だ",	""},
+{"ち",	"ぢ",	""},
+{"つ",	"づ",	""},
+{"て",	"で",	""},
+{"と",	"ど",	""},
+{"は",	"ば",	"ぱ"},
+{"ひ",	"び",	"ぴ"},
+{"ふ",	"ぶ",	"ぷ"},
+{"へ",	"べ",	"ぺ"},
+{"ほ",	"ぼ",	"ぽ"},
+{NULL, NULL, NULL},
+};
+
+KeyCodeToCharRule scim_anthy_keypad_table[] = {
+{SCIM_KEY_KP_Equal,		"="},
+{SCIM_KEY_KP_Multiply,	"*"},
+{SCIM_KEY_KP_Add,		"+"},
+{SCIM_KEY_KP_Separator,	","},
+{SCIM_KEY_KP_Subtract,	"-"},
+{SCIM_KEY_KP_Decimal,	"."},
+{SCIM_KEY_KP_Divide,	"/"},
+{SCIM_KEY_KP_0,			"0"},
+{SCIM_KEY_KP_1,			"1"},
+{SCIM_KEY_KP_2,			"2"},
+{SCIM_KEY_KP_3,			"3"},
+{SCIM_KEY_KP_4,			"4"},
+{SCIM_KEY_KP_5,			"5"},
+{SCIM_KEY_KP_6,			"6"},
+{SCIM_KEY_KP_7,			"7"},
+{SCIM_KEY_KP_8,			"8"},
+{SCIM_KEY_KP_9,			"9"},
+{0, NULL}
+};
+
+KeyCodeToCharRule scim_anthy_kana_table[] = {
+{SCIM_KEY_kana_fullstop,        "。"},
+{SCIM_KEY_kana_openingbracket,  "「"},
+{SCIM_KEY_kana_closingbracket,  "」"},
+{SCIM_KEY_kana_comma,           "、"},
+{SCIM_KEY_kana_conjunctive,     "・"},
+{SCIM_KEY_kana_middledot,       "・"},
+{SCIM_KEY_kana_WO,              "を"},
+{SCIM_KEY_kana_a,               "ぁ"},
+{SCIM_KEY_kana_i,               "ぃ"},
+{SCIM_KEY_kana_u,               "ぅ"},
+{SCIM_KEY_kana_e,               "ぇ"},
+{SCIM_KEY_kana_o,               "ぉ"},
+{SCIM_KEY_kana_ya,              "ゃ"},
+{SCIM_KEY_kana_yu,              "ゅ"},
+{SCIM_KEY_kana_yo,              "ょ"},
+{SCIM_KEY_kana_tsu,             "っ"},
+{SCIM_KEY_kana_tu,              "っ"},
+{SCIM_KEY_prolongedsound,       "ー"},
+{SCIM_KEY_kana_A,               "あ"},
+{SCIM_KEY_kana_I,               "い"},
+{SCIM_KEY_kana_U,               "う"},
+{SCIM_KEY_kana_E,               "え"},
+{SCIM_KEY_kana_O,               "お"},
+{SCIM_KEY_kana_O,               "お"},
+{SCIM_KEY_kana_KA,              "か"},
+{SCIM_KEY_kana_KI,              "き"},
+{SCIM_KEY_kana_KU,              "く"},
+{SCIM_KEY_kana_KE,              "け"},
+{SCIM_KEY_kana_KO,              "こ"},
+{SCIM_KEY_kana_SA,              "さ"},
+{SCIM_KEY_kana_SHI,             "し"},
+{SCIM_KEY_kana_SU,              "す"},
+{SCIM_KEY_kana_SE,              "せ"},
+{SCIM_KEY_kana_SO,              "そ"},
+{SCIM_KEY_kana_TA,              "た"},
+{SCIM_KEY_kana_CHI,             "ち"},
+{SCIM_KEY_kana_TI,              "ち"},
+{SCIM_KEY_kana_TSU,             "つ"},
+{SCIM_KEY_kana_TU,              "つ"},
+{SCIM_KEY_kana_TE,              "て"},
+{SCIM_KEY_kana_TO,              "と"},
+{SCIM_KEY_kana_NA,              "な"},
+{SCIM_KEY_kana_NI,              "に"},
+{SCIM_KEY_kana_NU,              "ぬ"},
+{SCIM_KEY_kana_NE,              "ね"},
+{SCIM_KEY_kana_NO,              "の"},
+{SCIM_KEY_kana_HA,              "は"},
+{SCIM_KEY_kana_HI,              "ひ"},
+{SCIM_KEY_kana_FU,              "ふ"},
+{SCIM_KEY_kana_HU,              "ふ"},
+{SCIM_KEY_kana_HE,              "へ"},
+{SCIM_KEY_kana_HO,              "ほ"},
+{SCIM_KEY_kana_MA,              "ま"},
+{SCIM_KEY_kana_MI,              "み"},
+{SCIM_KEY_kana_MU,              "む"},
+{SCIM_KEY_kana_ME,              "め"},
+{SCIM_KEY_kana_MO,              "も"},
+{SCIM_KEY_kana_YA,              "や"},
+{SCIM_KEY_kana_YU,              "ゆ"},
+{SCIM_KEY_kana_YO,              "よ"},
+{SCIM_KEY_kana_RA,              "ら"},
+{SCIM_KEY_kana_RI,              "り"},
+{SCIM_KEY_kana_RU,              "る"},
+{SCIM_KEY_kana_RE,              "れ"},
+{SCIM_KEY_kana_RO,              "ろ"},
+{SCIM_KEY_kana_WA,              "わ"},
+{SCIM_KEY_kana_N,               "ん"},
+{SCIM_KEY_voicedsound,          "゛"},
+{SCIM_KEY_semivoicedsound,      "゜"},
+{0, NULL}
+};
+
+NicolaRule scim_anthy_nicola_table[] = {
+// no modifiered keys
+{"1",	"1",	"？",	"！"},
+{"2",	"2",	"／",	"゛"},
+{"3",	"3",	"〜",	"＃"},
+{"4",	"4",	"「",	"＄"},
+{"5",	"5",	"」",	"％"},
+{"6",	"6",	"＆",	"［"},
+{"7",	"7",	"’",	"］"},
+{"8",	"8",	"（",	"（"},
+{"9",	"9",	"）",	"）"},
+{"0",	"0",	"｛",	"｛"},
+{"-",	"−",	"｝",	"｝"},
+{"^",	"｜",	"￥",	"￥"},
+{"\\",	"￥",	"￥",	"￥"},
+
+{"q",	"。",	"ぁ",	"ゐ"},
+{"w",	"か",	"え",	"が"},
+{"e",	"た",	"り",	"だ"},
+{"r",	"こ",	"ゃ",	"ご"},
+{"t",	"さ",	"れ",	"ざ"},
+{"y",	"ら",	"ぱ",	"よ"},
+{"u",	"ち",	"ぢ",	"に"},
+{"i",	"く",	"ぐ",	"る"},
+{"o",	"つ",	"づ",	"ま"},
+{"p",	"，",	"ぴ",	"ぇ"},
+{"@",	"、",	"゛",	"ー"},
+{"[",	"「",	"『",	"『"},
+
+{"a",	"う",	"を",	"ヴ"},
+{"s",	"し",	"あ",	"じ"},
+{"d",	"て",	"な",	"で"},
+{"f",	"け",	"ゅ",	"げ"},
+{"g",	"せ",	"も",	"ぜ"},
+{"h",	"は",	"ば",	"み"},
+{"j",	"と",	"ど",	"お"},
+{"k",	"き",	"ぎ",	"の"},
+{"l",	"い",	"ぽ",	"ょ"},
+{";",	"ん",	"；",	"っ"},
+{":",	"：",	"：",	"："},
+{"]",	"」",	"』",	"』"},
+
+{"z",	"．",	"ぅ",	"ゑ"},
+{"x",	"ひ",	"ー",	"び"},
+{"c",	"す",	"ろ",	"ず"},
+{"v",	"ふ",	"や",	"ぶ"},
+{"b",	"へ",	"ぃ",	"べ"},
+{"n",	"め",	"ぷ",	"ぬ"},
+{"m",	"そ",	"ぞ",	"ゆ"},
+{",",	"ね",	"ぺ",	"む"},
+{".",	"ほ",	"ぼ",	"わ"},
+{"/",	"・",	"ゎ",	"ぉ"},
+
+// normal shift modifiered keys
+{"!",	"！",	"",	""},
+{"\"",	"”",	"",	""},
+{"#",	"＃",	"",	""},
+{"$",	"＄",	"",	""},
+{"%",	"％",	"",	""},
+{"&",	"＆",	"",	""},
+{"'",	"’",	"",	""},
+{"(",	"（",	"",	""},
+{")",	"）",	"",	""},
+{"~",	"〜",	"",	""},
+{"=",	"＝",	"",	""},
+{"|",	"｜",	"",	""},
+
+{"`",	"‘",	"",	""},
+{"{",	"｛",	"",	""},
+
+{"+",	"＋",	"",	""},
+{"*",	"＊",	"",	""},
+{"}",	"｝",	"",	""},
+
+{"<",	"＜",	"",	""},
+{">",	"＞",	"",	""},
+{"?",	"？",	"",	""},
+{"_",	"＿",	"",	""},
+
+{NULL,	NULL,	NULL,	NULL},
+};
