@@ -60,6 +60,7 @@
 #define SCIM_PROP_TYPING_METHOD              "/IMEngine/Anthy/TypingMethod"
 #define SCIM_PROP_TYPING_METHOD_ROMAJI       "/IMEngine/Anthy/TypingMethod/RomaKana"
 #define SCIM_PROP_TYPING_METHOD_KANA         "/IMEngine/Anthy/TypingMethod/Kana"
+#define SCIM_PROP_TYPING_METHOD_NICOLA       "/IMEngine/Anthy/TypingMethod/NICOLA"
 
 #define SCIM_PROP_PERIOD_STYLE               "/IMEngine/Anthy/PeriodType"
 #define SCIM_PROP_PERIOD_STYLE_JAPANESE      "/IMEngine/Anthy/PeriodType/Japanese"
@@ -77,10 +78,11 @@ AnthyInstance::AnthyInstance (AnthyFactory   *factory,
                               int             id)
     : IMEngineInstanceBase     (factory, encoding, id),
       m_factory                (factory),
-      m_prev_key               (0,0),
-      m_preedit                (m_key2kana_tables),
+      m_on_init                (true),
+      m_preedit                (*this, m_key2kana_tables),
       m_preedit_string_visible (false),
       m_lookup_table_visible   (false),
+      m_n_conv_key_pressed     (0),
       m_prev_input_mode        (SCIM_ANTHY_MODE_HIRAGANA),
       m_conv_mode              (SCIM_ANTHY_CONVERSION_MULTI_SEGMENT)
 {
@@ -88,6 +90,7 @@ AnthyInstance::AnthyInstance (AnthyFactory   *factory,
 
     reload_config (m_factory->m_config);
     m_factory->append_config_listener (this);
+    m_on_init = false;
 }
 
 AnthyInstance::~AnthyInstance ()
@@ -95,6 +98,62 @@ AnthyInstance::~AnthyInstance ()
     m_factory->remove_config_listener (this);
 }
 
+// FIXME!
+bool
+AnthyInstance::is_nicola_thumb_shift_key (const KeyEvent &key)
+{
+    if (m_preedit.get_typing_method () != SCIM_ANTHY_TYPING_METHOD_NICOLA)
+        return false;
+
+    if (util_match_key_event (m_factory->m_left_thumb_keys, key, 0xFFFF) ||
+        util_match_key_event (m_factory->m_right_thumb_keys, key, 0xFFFF))
+    {
+        return true;
+    }
+
+    return false;
+}
+
+bool
+AnthyInstance::process_key_event_input (const KeyEvent &key)
+{
+    if (m_preedit.can_process_key_event (key)) {
+        if (m_preedit.is_converting ()) {
+            if (is_realtime_conversion ()) {
+                action_revert ();
+            } else if (!is_nicola_thumb_shift_key (key)) {
+                action_commit (m_factory->m_learn_on_auto_commit);
+            }
+        }
+
+        bool need_commit = m_preedit.process_key_event (key);
+
+        if (need_commit) {
+            if (is_realtime_conversion () &&
+                m_preedit.get_input_mode () != SCIM_ANTHY_MODE_LATIN &&
+                m_preedit.get_input_mode () != SCIM_ANTHY_MODE_WIDE_LATIN)
+            {
+                m_preedit.convert (SCIM_ANTHY_CANDIDATE_NORMAL,
+                                   is_single_segment ());
+            }
+            action_commit (m_factory->m_learn_on_auto_commit);
+        } else {
+            if (is_realtime_conversion ()) {
+                m_preedit.convert (SCIM_ANTHY_CANDIDATE_NORMAL,
+                                   is_single_segment ());
+                m_preedit.select_segment (-1);
+            }
+            show_preedit_string ();
+            m_preedit_string_visible = true;
+            set_preedition ();
+        }
+
+        return true;
+    }
+
+    return false;
+}
+
 bool
 AnthyInstance::process_key_event_lookup_keybind (const KeyEvent& key)
 {
@@ -113,75 +172,29 @@ AnthyInstance::process_key_event_lookup_keybind (const KeyEvent& key)
 bool
 AnthyInstance::process_key_event_without_preedit (const KeyEvent& key)
 {
-    return process_remaining_key_event (key);
+    return false;
 }
 
 bool
 AnthyInstance::process_key_event_with_preedit (const KeyEvent& key)
 {
-    return process_remaining_key_event (key);
+    return false;
 }
 
 bool
 AnthyInstance::process_key_event_with_candidate (const KeyEvent &key)
 {
-    return process_remaining_key_event (key);
-}
-
-bool
-AnthyInstance::process_remaining_key_event (const KeyEvent &key)
-{
-    if (m_preedit.can_process_key_event (key)) {
-        if (m_preedit.is_converting ()) {
-            if (is_realtime_conversion ())
-                action_revert ();
-            else
-                action_commit (m_factory->m_learn_on_auto_commit);
-        }
-
-        bool need_commit = m_preedit.process_key_event (key);
-
-        if (need_commit) {
-            if (is_realtime_conversion ())
-                m_preedit.convert (SCIM_ANTHY_CANDIDATE_NORMAL,
-                                   is_single_segment ());
-            action_commit (m_factory->m_learn_on_auto_commit);
-        } else {
-            if (is_realtime_conversion ()) {
-                m_preedit.convert (SCIM_ANTHY_CANDIDATE_NORMAL,
-                                   is_single_segment ());
-                m_preedit.select_segment (-1);
-            }
-            show_preedit_string ();
-            m_preedit_string_visible = true;
-            set_preedition ();
-        }
-
-        return true;
-    }
-
-    if (m_preedit.is_preediting ())
-        return true;
-    else
-        return false;
+    return false;
 }
 
 bool
 AnthyInstance::process_key_event (const KeyEvent& key)
 {
     SCIM_DEBUG_IMENGINE(2) << "process_key_event.\n";
-    KeyEvent newkey;
 
-    // ignore key release.
-    if (key.is_key_release ())
+    if (process_key_event_input (key))
         return true;
 
-    // ignore modifier keys
-    if (key.code == SCIM_KEY_Shift_L || key.code == SCIM_KEY_Shift_R ||
-        key.code == SCIM_KEY_Control_L || key.code == SCIM_KEY_Control_R ||
-        key.code == SCIM_KEY_Alt_L || key.code == SCIM_KEY_Alt_R)
-        return false;
-
     // lookup user defined key bindings
     if (process_key_event_lookup_keybind (key))
         return true;
@@ -190,12 +203,20 @@ AnthyInstance::process_key_event (const KeyEvent& key)
         return false;
 
     // process hard coded keys
-    if (is_selecting_candidates ())
-        return process_key_event_with_candidate (key);
+    if (is_selecting_candidates () && m_lookup_table_visible)
+        if (process_key_event_with_candidate (key))
+            return true;
     else if (m_preedit.is_preediting ())
-        return process_key_event_with_preedit(key);
+        if (process_key_event_with_preedit(key))
+            return true;
     else
-        return process_key_event_without_preedit(key);
+        if (process_key_event_without_preedit(key))
+            return true;
+
+    if (m_preedit.is_preediting ())
+        return true;
+    else
+        return false;
 }
 
 void
@@ -211,7 +232,7 @@ AnthyInstance::select_candidate_no_direct (unsigned int item)
     if (!is_selecting_candidates ()) return;
 
     SCIM_DEBUG_IMENGINE(2) << "select_candidate_no_direct.\n";
- 
+
     // update lookup table
     m_lookup_table.set_cursor_pos_in_current_page (item);
     update_lookup_table (m_lookup_table);
@@ -219,6 +240,15 @@ AnthyInstance::select_candidate_no_direct (unsigned int item)
     // update preedit
     m_preedit.select_candidate (m_lookup_table.get_cursor_pos ());
     set_preedition ();
+
+    // update aux string
+    if (m_factory->m_show_candidates_label) {
+        char buf[256];
+        sprintf (buf, _("Candidates (%d/%d)"),
+                 m_lookup_table.get_cursor_pos () + 1,
+                 m_lookup_table.number_of_candidates ());
+        update_aux_string (utf8_mbstowcs (buf));
+    }
 }
 
 void
@@ -229,9 +259,7 @@ AnthyInstance::select_candidate (unsigned int item)
     select_candidate_no_direct (item);
 
     if (m_factory->m_close_cand_win_on_select) {
-        m_lookup_table.clear ();
-        hide_lookup_table ();
-        m_lookup_table_visible = false;
+        unset_lookup_table ();
         action_select_next_segment();
     }
 }
@@ -285,12 +313,10 @@ AnthyInstance::reset ()
     m_preedit.clear ();
     m_lookup_table.clear ();
 
-    hide_lookup_table ();
+    unset_lookup_table ();
+
     hide_preedit_string ();
-
     m_preedit_string_visible = false;
-    m_lookup_table_visible   = false;
-
     set_preedition ();
 }
 
@@ -332,6 +358,68 @@ AnthyInstance::set_preedition (void)
     update_preedit_caret (m_preedit.get_caret_pos());
 }
 
+void
+AnthyInstance::set_lookup_table (void)
+{
+    m_n_conv_key_pressed++;
+
+    if (!is_selecting_candidates ()) {
+        if (is_realtime_conversion () &&
+            m_preedit.get_selected_segment () < 0)
+        {
+            // select latest segment
+            int n = m_preedit.get_nr_segments ();
+            if (n < 1)
+                return;
+            m_preedit.select_segment (n - 1);
+        }
+
+        // prepare candidates
+        m_preedit.get_candidates (m_lookup_table);
+
+        if (m_lookup_table.number_of_candidates () == 0)
+            return;
+
+        // set position
+        update_lookup_table (m_lookup_table);
+
+        // update preedit
+        m_preedit.select_candidate (m_lookup_table.get_cursor_pos ());
+        set_preedition ();
+
+    }
+
+    if (!m_lookup_table_visible &&
+        m_factory->m_n_triggers_to_show_cand_win > 0 &&
+        (int) m_n_conv_key_pressed >= m_factory->m_n_triggers_to_show_cand_win)
+    {
+        show_lookup_table ();
+        m_lookup_table_visible = true;
+        m_n_conv_key_pressed = 0;
+
+        if (m_factory->m_show_candidates_label) {
+            char buf[256];
+            sprintf (buf, _("Candidates (%d/%d)"),
+                     m_lookup_table.get_cursor_pos () + 1,
+                     m_lookup_table.number_of_candidates ());
+            update_aux_string (utf8_mbstowcs (buf));
+            show_aux_string ();
+        }
+    }
+}
+
+void
+AnthyInstance::unset_lookup_table (void)
+{
+    m_lookup_table.clear ();
+    hide_lookup_table ();
+    m_lookup_table_visible = false;
+    m_n_conv_key_pressed = 0;
+
+    update_aux_string (utf8_mbstowcs (""));
+    hide_aux_string ();
+}
+
 void
 AnthyInstance::install_properties (void)
 {
@@ -377,6 +465,10 @@ AnthyInstance::install_properties (void)
             prop = Property (SCIM_PROP_TYPING_METHOD_KANA,
                              _("Kana"), String (""), _("Kana"));
             m_properties.push_back (prop);
+
+            prop = Property (SCIM_PROP_TYPING_METHOD_NICOLA,
+                             _("Thumb shift"), String (""), _("Thumb shift"));
+            m_properties.push_back (prop);
         }
 
         if (m_factory->m_show_conv_mode_label) {
@@ -556,6 +648,9 @@ AnthyInstance::set_typing_method (TypingMethod method)
     case SCIM_ANTHY_TYPING_METHOD_KANA:
         label = "\xE3\x81\x8B";
         break;
+    case SCIM_ANTHY_TYPING_METHOD_NICOLA:
+        label = "\xE8\xA6\xAA";
+        break;
     default:
         break;
     }
@@ -574,7 +669,10 @@ AnthyInstance::set_typing_method (TypingMethod method)
         Key2KanaTable *fundamental_table = NULL;
         if (method == SCIM_ANTHY_TYPING_METHOD_ROMAJI)
             fundamental_table = m_factory->m_custom_romaji_table;
-        m_preedit.set_typing_method (method, fundamental_table);
+        else if (method == SCIM_ANTHY_TYPING_METHOD_KANA)
+            fundamental_table = m_factory->m_custom_kana_table;
+        m_preedit.set_typing_method
+            (method, fundamental_table);
     }
 }
 
@@ -637,6 +735,12 @@ AnthyInstance::is_selecting_candidates (void)
         return false;
 }
 
+bool
+AnthyInstance::action_do_nothing (void)
+{
+    return true;
+}
+
 bool
 AnthyInstance::action_convert (void)
 {
@@ -649,50 +753,11 @@ AnthyInstance::action_convert (void)
         m_preedit.convert (SCIM_ANTHY_CANDIDATE_NORMAL,
                            is_single_segment ());
         set_preedition ();
-    } else {
-        if (is_realtime_conversion () &&
-            m_preedit.get_selected_segment () < 0)
-        {
-            int n = m_preedit.get_nr_segments ();
-            if (n < 1)
-                return false;
-            m_preedit.select_segment (n - 1);
-        }
-
-        // show candidates window
-        m_preedit.get_candidates (m_lookup_table);
-
-        if (m_lookup_table.number_of_candidates () == 0)
-            return false;
-
-        int selected = m_preedit.get_selected_candidate ();
-        int page_size = m_lookup_table.get_page_size ();
-        int end = m_lookup_table.number_of_candidates () - 1;
-
-        int page = selected / page_size;
-        int pos  = selected % page_size;
-
-        if (m_preedit.get_selected_candidate () >= end) {
-            m_lookup_table.set_cursor_pos (0);
-            select_candidate_no_direct (0);
-
-        } else {
-            for (int i = 0; i < page; i++)
-                m_lookup_table.page_down ();
-
-            if (pos + 1 >= m_lookup_table.get_page_size ()) {
-                m_lookup_table.page_down ();
-                select_candidate_no_direct (0);
-
-            } else {
-                select_candidate_no_direct (pos + 1);
-            }
-        }
-        show_lookup_table ();
-        m_lookup_table_visible = true;
+        set_lookup_table ();
+        return true;
     }
-    
-    return true;
+
+    return false;
 }
 
 bool
@@ -708,14 +773,13 @@ AnthyInstance::action_revert (void)
 
     if (is_selecting_candidates ()) {
         m_lookup_table.clear ();
-        hide_lookup_table ();
-        m_lookup_table_visible = false;
-        return true;
+        if (m_lookup_table_visible) {
+            unset_lookup_table ();
+            return true;
+        }
     }
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     m_preedit.revert ();
     set_preedition ();
@@ -953,9 +1017,7 @@ AnthyInstance::action_select_prev_segment (void)
     if (!m_preedit.is_converting ())
         return false;
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     int idx = m_preedit.get_selected_segment ();
     if (idx - 1 < 0) {
@@ -976,9 +1038,7 @@ AnthyInstance::action_select_next_segment (void)
     if (!m_preedit.is_converting ())
         return false;
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     int idx = m_preedit.get_selected_segment ();
     if (idx < 0) {
@@ -1003,9 +1063,7 @@ AnthyInstance::action_select_first_segment (void)
     if (!m_preedit.is_converting ())
         return false;
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     m_preedit.select_segment(0);
     set_preedition ();
@@ -1022,9 +1080,7 @@ AnthyInstance::action_select_last_segment (void)
     int n = m_preedit.get_nr_segments ();
     if (n <= 0) return false;
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     m_preedit.select_segment(n - 1);
     set_preedition ();
@@ -1038,9 +1094,7 @@ AnthyInstance::action_shrink_segment (void)
     if (!m_preedit.is_converting ())
         return false;
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     m_preedit.resize_segment (-1);
     set_preedition ();
@@ -1054,9 +1108,7 @@ AnthyInstance::action_expand_segment (void)
     if (!m_preedit.is_converting ())
         return false;
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     m_preedit.resize_segment (1);
     set_preedition ();
@@ -1075,9 +1127,7 @@ AnthyInstance::action_commit_first_segment (void)
         }
     }
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     commit_string (m_preedit.get_segment_string (0));
     if (m_factory->m_learn_on_manual_commit)
@@ -1099,9 +1149,7 @@ AnthyInstance::action_commit_selected_segment (void)
         }
     }
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     for (int i = 0; i <= m_preedit.get_selected_segment (); i++)
         commit_string (m_preedit.get_segment_string (i));
@@ -1124,9 +1172,7 @@ AnthyInstance::action_commit_first_segment_reverse_preference (void)
         }
     }
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     commit_string (m_preedit.get_segment_string (0));
     if (!m_factory->m_learn_on_manual_commit)
@@ -1148,9 +1194,7 @@ AnthyInstance::action_commit_selected_segment_reverse_preference (void)
         }
     }
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     for (int i = 0; i <= m_preedit.get_selected_segment (); i++)
         commit_string (m_preedit.get_segment_string (i));
@@ -1168,10 +1212,8 @@ AnthyInstance::action_select_next_candidate (void)
     if (!m_preedit.is_converting ())
         return false;
 
-    if (!is_selecting_candidates ()) {
-        action_convert ();
-        return true;
-    }
+    //if (!is_selecting_candidates ())
+        set_lookup_table ();
 
     int end = m_lookup_table.number_of_candidates () - 1;
     if (m_lookup_table.get_cursor_pos () == end)
@@ -1192,8 +1234,8 @@ AnthyInstance::action_select_prev_candidate (void)
 {
     if (!m_preedit.is_converting ()) return false;
 
-    if (!is_selecting_candidates ())
-        action_convert ();
+    //if (!is_selecting_candidates ())
+        set_lookup_table ();
 
     int end = m_lookup_table.number_of_candidates () - 1;
     if (m_lookup_table.get_cursor_pos () == 0)
@@ -1373,8 +1415,10 @@ AnthyInstance::action_circle_typing_method (void)
     TypingMethod method;
 
     method = m_preedit.get_typing_method ();
-    if (method == SCIM_ANTHY_TYPING_METHOD_KANA)
+    if (method == SCIM_ANTHY_TYPING_METHOD_NICOLA)
         method = SCIM_ANTHY_TYPING_METHOD_ROMAJI;
+    else if (method == SCIM_ANTHY_TYPING_METHOD_KANA)
+        method = SCIM_ANTHY_TYPING_METHOD_NICOLA;
     else
         method = SCIM_ANTHY_TYPING_METHOD_KANA;
 
@@ -1464,9 +1508,7 @@ AnthyInstance::convert_kana (CandidateType type)
     if (!m_preedit.is_preediting ())
         return false;
 
-    m_lookup_table.clear ();
-    hide_lookup_table ();
-    m_lookup_table_visible = false;
+    unset_lookup_table ();
 
     if (m_preedit.is_converting ()) {
         int idx = m_preedit.get_selected_segment ();
@@ -1526,7 +1568,7 @@ AnthyInstance::action_convert_to_wide_latin (void)
 bool
 AnthyInstance::action_add_word (void)
 {
-    launch_program (m_factory->m_add_word_command.c_str ());
+    util_launch_program (m_factory->m_add_word_command.c_str ());
 
     return true;
 }
@@ -1534,7 +1576,7 @@ AnthyInstance::action_add_word (void)
 bool
 AnthyInstance::action_launch_dict_admin_tool (void)
 {
-    launch_program (m_factory->m_dict_admin_command.c_str ());
+    util_launch_program (m_factory->m_dict_admin_command.c_str ());
 
     return true;
 }
@@ -1581,6 +1623,8 @@ AnthyInstance::trigger_property (const String &property)
         set_typing_method (SCIM_ANTHY_TYPING_METHOD_ROMAJI);
     } else if (property == SCIM_PROP_TYPING_METHOD_KANA) {
         set_typing_method (SCIM_ANTHY_TYPING_METHOD_KANA);
+    } else if (property == SCIM_PROP_TYPING_METHOD_NICOLA) {
+        set_typing_method (SCIM_ANTHY_TYPING_METHOD_NICOLA);
 
     // period type
     } else if (property == SCIM_PROP_PERIOD_STYLE_JAPANESE) {
@@ -1613,54 +1657,64 @@ AnthyInstance::reload_config (const ConfigPointer &config)
     m_preedit.set_allow_split_romaji (m_factory->m_romaji_allow_split);
 
     // set input mode
-    if (m_factory->m_input_mode == "Hiragana")
-        m_preedit.set_input_mode (SCIM_ANTHY_MODE_HIRAGANA);
-    else if (m_factory->m_input_mode == "Katakana")
-        m_preedit.set_input_mode (SCIM_ANTHY_MODE_KATAKANA);
-    else if (m_factory->m_input_mode == "HalfKatakana")
-        m_preedit.set_input_mode (SCIM_ANTHY_MODE_HALF_KATAKANA);
-    else if (m_factory->m_input_mode == "Latin")
-        m_preedit.set_input_mode (SCIM_ANTHY_MODE_LATIN);
-    else if (m_factory->m_input_mode == "WideLatin")
-        m_preedit.set_input_mode (SCIM_ANTHY_MODE_WIDE_LATIN);
+    if (m_on_init || !m_factory->m_show_input_mode_label) {
+        if (m_factory->m_input_mode == "Hiragana")
+            m_preedit.set_input_mode (SCIM_ANTHY_MODE_HIRAGANA);
+        else if (m_factory->m_input_mode == "Katakana")
+            m_preedit.set_input_mode (SCIM_ANTHY_MODE_KATAKANA);
+        else if (m_factory->m_input_mode == "HalfKatakana")
+            m_preedit.set_input_mode (SCIM_ANTHY_MODE_HALF_KATAKANA);
+        else if (m_factory->m_input_mode == "Latin")
+            m_preedit.set_input_mode (SCIM_ANTHY_MODE_LATIN);
+        else if (m_factory->m_input_mode == "WideLatin")
+            m_preedit.set_input_mode (SCIM_ANTHY_MODE_WIDE_LATIN);
+    }
 
     // set typing method
-    if (m_factory->m_typing_method == "Kana")
-        m_preedit.set_typing_method (SCIM_ANTHY_TYPING_METHOD_KANA,
-                                     NULL);
-    else if (m_factory->m_typing_method == "Roma")
-        m_preedit.set_typing_method (SCIM_ANTHY_TYPING_METHOD_ROMAJI,
-                                     m_factory->m_custom_romaji_table);
-    else
-        m_preedit.set_typing_method (SCIM_ANTHY_TYPING_METHOD_ROMAJI,
-                                     m_factory->m_custom_romaji_table);
+    if (m_on_init || !m_factory->m_show_typing_method_label) {
+        if (m_factory->m_typing_method == "NICOLA")
+            m_preedit.set_typing_method
+                (SCIM_ANTHY_TYPING_METHOD_NICOLA, NULL);
+        else if (m_factory->m_typing_method == "Kana")
+            m_preedit.set_typing_method
+                (SCIM_ANTHY_TYPING_METHOD_KANA,
+                 m_factory->m_custom_kana_table);
+        else
+            m_preedit.set_typing_method
+                (SCIM_ANTHY_TYPING_METHOD_ROMAJI,
+                 m_factory->m_custom_romaji_table);
+    }
 
     // set conversion mode
-    if (m_factory->m_conversion_mode == "MultiSeg")
-        m_conv_mode = SCIM_ANTHY_CONVERSION_MULTI_SEGMENT;
-    else if (m_factory->m_conversion_mode == "SingleSeg")
-        m_conv_mode = SCIM_ANTHY_CONVERSION_SINGLE_SEGMENT;
-    else if (m_factory->m_conversion_mode == "CAYT_MultiSeg")
-        m_conv_mode = SCIM_ANTHY_CONVERSION_MULTI_SEGMENT_IMMEDIATE;
-    else if (m_factory->m_conversion_mode == "CAYT_SingleSeg")
-        m_conv_mode = SCIM_ANTHY_CONVERSION_SINGLE_SEGMENT_IMMEDIATE;
+    if (m_on_init || !m_factory->m_show_conv_mode_label) {
+        if (m_factory->m_conversion_mode == "MultiSeg")
+            m_conv_mode = SCIM_ANTHY_CONVERSION_MULTI_SEGMENT;
+        else if (m_factory->m_conversion_mode == "SingleSeg")
+            m_conv_mode = SCIM_ANTHY_CONVERSION_SINGLE_SEGMENT;
+        else if (m_factory->m_conversion_mode == "CAYT_MultiSeg")
+            m_conv_mode = SCIM_ANTHY_CONVERSION_MULTI_SEGMENT_IMMEDIATE;
+        else if (m_factory->m_conversion_mode == "CAYT_SingleSeg")
+            m_conv_mode = SCIM_ANTHY_CONVERSION_SINGLE_SEGMENT_IMMEDIATE;
+    }
 
     // set period style
-    if (m_factory->m_period_style == "WideLatin") {
-        m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_WIDE);
-        m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_WIDE);
-    } else if (m_factory->m_period_style == "Latin") {
-        m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_HALF);
-        m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_HALF);
-    } else if (m_factory->m_period_style == "Japanese") {
-        m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_JAPANESE);
-        m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
-    } else if (m_factory->m_period_style == "WideLatin_Japanese") {
-        m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_WIDE);
-        m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
-    } else {
-        m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_JAPANESE);
-        m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
+    if (m_on_init || !m_factory->m_show_period_style_label) {
+        if (m_factory->m_period_style == "WideLatin") {
+            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_WIDE);
+            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_WIDE);
+        } else if (m_factory->m_period_style == "Latin") {
+            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_HALF);
+            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_HALF);
+        } else if (m_factory->m_period_style == "Japanese") {
+            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_JAPANESE);
+            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
+        } else if (m_factory->m_period_style == "WideLatin_Japanese") {
+            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_WIDE);
+            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
+        } else {
+            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_JAPANESE);
+            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
+        }
     }
 
     // set ten key type
@@ -1679,6 +1733,12 @@ AnthyInstance::reload_config (const ConfigPointer &config)
     else
         m_preedit.set_behavior_on_period (SCIM_ANTHY_NONE_ON_PERIOD);
 
+    // set lookup table
+    if (m_factory->m_cand_win_page_size > 0)
+        m_lookup_table.set_page_size (m_factory->m_cand_win_page_size);
+    else
+        m_lookup_table.set_page_size (SCIM_ANTHY_CONFIG_CAND_WIN_PAGE_SIZE_DEFAULT);
+
     // setup toolbar
     m_properties.clear ();
     install_properties ();
