@@ -26,7 +26,8 @@
 #ifndef __SCIM_ANTHY_IMENGINE_H__
 #define __SCIM_ANTHY_IMENGINE_H__
 
-#define Uses_SCIM_ICONV
+#define Uses_SCIM_IMENGINE
+#define Uses_SCIM_CONFIG_BASE
 #include <anthy/anthy.h>
 #include <scim.h>
 #include "scim_anthy_preedit.h"
@@ -35,6 +36,8 @@
 using namespace scim;
 using namespace scim_anthy;
 
+class AnthyFactory;
+
 typedef enum {
     SCIM_ANTHY_CONVERSION_MULTI_SEGMENT,
     SCIM_ANTHY_CONVERSION_SINGLE_SEGMENT,
@@ -64,7 +67,9 @@ public:
     virtual void reload_config                (const ConfigPointer &config);
 
 public:
-    /* actinos */
+    /* actions */
+    bool   action_do_nothing                  (void);
+
     bool   action_convert                     (void);
     bool   action_revert                      (void);
     bool   action_commit_follow_preference    (void);
@@ -133,16 +138,22 @@ public:
     void   actoin_register_word               (void);
     */
 
+public:
+    AnthyFactory *
+           get_factory                        (void) { return m_factory; };
+
 private:
     /* processing key event */
+    bool   process_key_event_input            (const KeyEvent &key);
     bool   process_key_event_lookup_keybind   (const KeyEvent &key);
     bool   process_key_event_without_preedit  (const KeyEvent &key);
     bool   process_key_event_with_preedit     (const KeyEvent &key);
     bool   process_key_event_with_candidate   (const KeyEvent &key);
-    bool   process_remaining_key_event        (const KeyEvent &key);
 
     /* utility */
     void   set_preedition                     (void);
+    void   set_lookup_table                   (void);
+    void   unset_lookup_table                 (void);
     void   install_properties                 (void);
     void   set_input_mode                     (InputMode      mode);
     void   set_conversion_mode                (ConversionMode mode);
@@ -159,10 +170,13 @@ private:
     bool   is_single_segment                  (void);
     bool   is_realtime_conversion             (void);
 
+private: // FIXME!
+    bool   is_nicola_thumb_shift_key          (const KeyEvent &key);
+
 private:
     AnthyFactory         *m_factory;
 
-    KeyEvent              m_prev_key;
+    bool                  m_on_init;
 
     /* for preedit */
     Key2KanaTableSet      m_key2kana_tables;
@@ -172,6 +186,7 @@ private:
     /* for candidates window */
     CommonLookupTable     m_lookup_table;
     bool                  m_lookup_table_visible;
+    unsigned int          m_n_conv_key_pressed;
 
     /* for toggling latin and wide latin */
     InputMode             m_prev_input_mode;
@@ -181,6 +196,9 @@ private:
 
     /*  */
     ConversionMode        m_conv_mode;
+
+    /* FIXME! */
+    bool                  m_forwarding_thumb_shift_key;
 };
 #endif /* __SCIM_ANTHY_IMENGINE_H__ */
 /*
