@@ -24,9 +24,13 @@
 #define Uses_SCIM_EVENT
 #include <scim.h>
 #include "scim_anthy_key2kana.h"
+#include "scim_anthy_kana.h"
+#include "scim_anthy_nicola.h"
 
 using namespace scim;
 
+class AnthyInstance;
+
 namespace scim_anthy {
 
 typedef std::vector<KeyEvent> KeyEvents;
@@ -39,12 +43,6 @@ typedef enum {
     SCIM_ANTHY_STRING_HALF_KATAKANA,
 } StringType;
 
-typedef enum {
-    SCIM_ANTHY_TEN_KEY_HALF,
-    SCIM_ANTHY_TEN_KEY_WIDE,
-    SCIM_ANTHY_TEN_KEY_FOLLOW_MODE,
-} TenKeyType;
-
 class Reading;
 class ReadingSegment;
 typedef std::vector<ReadingSegment> ReadingSegments;
@@ -66,13 +64,12 @@ private:
 //    KeyEvents  keys;
     String     raw;
     WideString kana;
-
 };
 
 class Reading
 {
 public:
-    Reading (Key2KanaTableSet & tables);
+    Reading (AnthyInstance &anthy, Key2KanaTableSet & tables);
     virtual ~Reading ();
 
     bool         can_process_key_event (const KeyEvent & key);
@@ -94,11 +91,6 @@ public:
     void         get_raw               (String     & string,
                                         unsigned int start  = 0,
                                         int          length = -1);
-#if 0
-    void         get_key_events        (KeyEvents  & events,
-                                        unsigned int start  = 0,
-                                        int          length = -1);
-#endif
     void         erase                 (unsigned int start  = 0,
                                         int          length = -1,
                                         bool         allow_split = false);
@@ -118,22 +110,23 @@ public:
     TypingMethod get_typing_method     (void);
 
 private:
-    bool         append_str            (const String & str);
     void         reset_pending         (void);
     void         split_segment         (unsigned int seg_id);
 
 private:
+    AnthyInstance         &m_anthy;
+
     // convertors
-    Key2KanaTableSet  &m_key2kana_tables;
-    Key2KanaConvertor  m_key2kana;
+    Key2KanaTableSet      &m_key2kana_tables;
+    Key2KanaConvertor      m_key2kana_normal;
+    KanaConvertor          m_kana;
+    NicolaConvertor        m_nicola;
+    Key2KanaConvertorBase *m_key2kana;
 
     // state
-    ReadingSegments    m_segments;
-    unsigned int       m_segment_pos;
-    unsigned int       m_caret_offset;
-
-    // mode
-    TenKeyType         m_ten_key_type;
+    ReadingSegments        m_segments;
+    unsigned int           m_segment_pos;
+    unsigned int           m_caret_offset;
 };
 
 }
