@@ -18,6 +18,7 @@
  */
 
 #include "scim_anthy_style_file.h"
+#include "scim_anthy_utils.h"
 
 using namespace scim_anthy;
 
@@ -30,10 +31,11 @@ escape (const String &str)
 
     for (unsigned int i = 0; i < dest.size (); i++) {
         if (dest[i] == '#'  ||                   // for comment
-            dest[i] == '\\' ||                   // for backslash it self
+            dest[i] == '\\' ||                   // for backslash itself
             dest[i] == '='  ||                   // for separatort
             dest[i] == '['  || dest[i] == ']' || // for section
-            dest[i] == '\t' || dest[i] == ',')   // for array
+            dest[i] == ','  ||                   // for array
+            dest[i] == ' '  || dest[i] == '\t')  // for space
         {
             dest.insert (i, "\\");
             i++;
@@ -59,13 +61,30 @@ unescape (const String &str)
     return dest;
 }
 
-StyleLine::StyleLine (StyleFile *style_file, const char *line)
+StyleLine::StyleLine (StyleFile *style_file, String line)
     : m_style_file (style_file),
       m_line  (line),
       m_type  (SCIM_ANTHY_STYLE_LINE_UNKNOWN)
 {
 }
 
+StyleLine::StyleLine (StyleFile *style_file, String key, String value)
+    : m_style_file (style_file),
+      m_line  (escape (key) + String ("=")),
+      m_type  (SCIM_ANTHY_STYLE_LINE_KEY)
+{
+    set_value (value);
+}
+
+StyleLine::StyleLine (StyleFile *style_file, String key,
+                      std::vector<String> &value)
+    : m_style_file (style_file),
+      m_line  (escape (key) + String("=")),
+      m_type  (SCIM_ANTHY_STYLE_LINE_KEY)
+{
+    set_value_array (value);
+}
+
 StyleLine::~StyleLine ()
 {
 }
@@ -166,61 +185,47 @@ StyleLine::get_key (String &key)
     return true;
 }
 
+static int
+get_value_position (String &str)
+{
+    unsigned int spos;
+    for (spos = 0;
+         spos < str.length ();
+         spos++)
+    {
+        if (str[spos] == '\\') {
+            spos++;
+            continue;
+        }
+        if (str[spos] == '=') {
+            break;
+        }
+    }
+    if (spos >= str.length ())
+        return true;
+    else
+        spos++;
+    for (;
+         spos < str.length () && isspace(str[spos]);
+         spos++);
+
+    return spos;
+}
+
 bool
 StyleLine::get_value (String &value)
 {
     if (get_type () != SCIM_ANTHY_STYLE_LINE_KEY)
         return false;
 
-    unsigned int spos, epos;
-    for (spos = 0;
-         spos < m_line.length ();
-         spos++)
-    {
-        if (m_line[spos] == '\\') {
-            spos++;
-            continue;
-        }
-        if (m_line[spos] == '=') {
-            break;
-        }
-    }
-    if (spos >= m_line.length ())
-        return true;
-    else
-        spos++;
-    for (;
-         spos < m_line.length () && isspace(m_line[spos]);
-         spos++);
-
-#if 0
-    // reduce traling space.
-    for (epos = m_line.length ();
-         epos >= spos && isspace (m_line[epos]);
-         epos--);
-    if (!isspace(m_line[epos]))
-        epos++;
-#else
-    // don't reduce trailing space.
-    epos = m_line.length ();
-#endif
+    unsigned int spos = get_value_position (m_line);
+    unsigned int epos = m_line.length ();
 
     value = unescape (m_line.substr (spos, epos - spos));
 
     return true;
 }
 
-bool
-StyleLine::get_value (WideString &value)
-{
-    String str;
-    bool success = get_value (str);
-    if (!success)
-        return false;
-    value = utf8_mbstowcs (str);
-    return true;
-}
-
 void
 StyleLine::set_value (String value)
 {
@@ -229,6 +234,51 @@ StyleLine::set_value (String value)
     m_line = escape (key) + String ("=") + escape (value);
 }
 
+bool
+StyleLine::get_value_array (std::vector<String> &value)
+{
+    if (get_type () != SCIM_ANTHY_STYLE_LINE_KEY)
+        return false;
+
+    unsigned int spos = get_value_position (m_line);
+    unsigned int epos = m_line.length ();
+
+    unsigned int head_of_element = spos;
+    for (unsigned int i = spos; i <= epos; i++) {
+        if (i < epos && m_line[i] == '\\') {
+            i++;
+            continue;
+        }
+
+        if (i == epos || m_line[i] == ',') {
+            String str;
+            if (head_of_element == epos)
+                str = String ();
+            else
+                str = unescape (m_line.substr (head_of_element,
+                                               i - head_of_element));
+            value.push_back (str);
+            head_of_element = i + 1;
+        }
+    }
+
+    return true;
+}
+
+void
+StyleLine::set_value_array (std::vector<String> &value)
+{
+    String key;
+    get_key (key);
+
+    m_line = escape (key) + String ("=");
+    for (unsigned int i = 0; i < value.size (); i++) {
+        if (i != 0)
+            m_line += ",";
+        m_line += escape (value[i]);
+    }
+}
+
 
 StyleFile::StyleFile ()
 {
@@ -258,8 +308,9 @@ StyleFile::load (const char *filename)
         if (in_file.eof ())
             break;
 
-        // FIXME! convert to UTF-8 or WideString
-        StyleLine line (this, buf);
+        WideString dest;
+        m_iconv.convert (dest, buf);
+        StyleLine line (this, utf8_wcstombs (dest));
         StyleLineType type = line.get_type ();
 
         if (type == SCIM_ANTHY_STYLE_LINE_SECTION) {
@@ -309,10 +360,10 @@ StyleFile::save (const char *filename)
     for (it = m_sections.begin (); it != m_sections.end (); it++) {
         StyleLines::iterator lit;
         for (lit = it->begin (); lit != it->end (); lit++) {
-            String line;
+            String line, dest;
             lit->get_line (line);
-            // FIXME! should convert to specified encoding.
-            out_file << line.c_str () << std::endl;
+            m_iconv.convert (dest, utf8_mbstowcs (line));
+            out_file << dest.c_str () << std::endl;
         }
     }
 
@@ -346,6 +397,12 @@ StyleFile::get_title (void)
     return m_title;
 }
 
+String
+StyleFile::get_file_name (void)
+{
+    return m_filename;
+}
+
 bool
 StyleFile::get_string (String &value, String section, String key)
 {
@@ -380,70 +437,129 @@ StyleFile::get_string (WideString &value, String section, String key)
     bool success = get_string (str, section, key);
     if (!success)
         return false;
-    // FIXME! should be converted on loading.
-    return m_iconv.convert (value, str);
+    value = utf8_mbstowcs (str);
+    return true;
+}
+
+bool
+StyleFile::get_string_array (std::vector<String> &value,
+                             String section, String key)
+{
+    StyleLines *lines = find_section (section);
+    if (!lines)
+        return false;
+
+    // find entry
+    StyleLines::iterator lit;
+    for (lit = lines->begin (); lit != lines->end (); lit++) {
+        String k;
+        lit->get_key (k);
+        if (k == key) {
+            lit->get_value_array (value);
+            return true;
+        }
+    }
+
+    return false;
+}
+
+bool
+StyleFile::get_string_array (std::vector<WideString> &value,
+                             String section, String key)
+{
+    std::vector<String> array;
+    bool success = get_string_array (array, section, key);
+    if (!success)
+        return false;
+
+    std::vector<String>::iterator it;
+    for (it = array.begin (); it != array.end (); it++)
+        value.push_back (utf8_mbstowcs (*it));
+    return true;
 }
 
 void
 StyleFile::set_string (String section, String key, String value)
 {
-    // find section
-    StyleSections::iterator it;
-    for (it = m_sections.begin (); it != m_sections.end (); it++) {
-        if (it->size () <= 0)
-            continue;
-
-        String s, k;
-        (*it)[0].get_section (s);
-
-        if (s != section)
-            continue;
-
+    StyleLines *lines = find_section (section);
+    if (lines) {
         // find entry
-        StyleLines::iterator lit, last = it->begin () + 1;
-        for (lit = last; lit != it->end (); lit++) {
+        StyleLines::iterator lit, last = lines->begin () + 1;
+        for (lit = last; lit != lines->end (); lit++) {
             StyleLineType type = lit->get_type ();
             if (type != SCIM_ANTHY_STYLE_LINE_SPACE)
-                last = lit;
+                last = lit + 1;
+
+            String k;
             lit->get_key (k);
 
             // replace existing entry
-            if (k == key) {
+            if (k.length () > 0 && k == key) {
                 lit->set_value (value);
                 return;
             }
         }
 
         // append new entry if no mathced entry exists.
-        String str = escape (key) + String ("=") + escape (value);
-        //it->insert (last + 1, StyleLine (this, str.c_str ()));
-        it->push_back (StyleLine (this, str.c_str ()));
-        return;
-    }
+        lines->insert (last, StyleLine (this, key, value));
 
-    // append space
-    if (!m_sections.empty()) {
-        StyleLines &sec = m_sections.back ();
-        if (sec.empty() ||
-            sec.back().get_type() != SCIM_ANTHY_STYLE_LINE_SPACE)
-        {
-            sec.push_back (StyleLine (this, ""));
+    } else {
+        StyleLines &newsec = append_new_section (section);
+
+        // append new entry
+        newsec.push_back (StyleLine (this, key, value));
+    }
+}
+
+void
+StyleFile::set_string (String section, String key, WideString value)
+{
+    set_string (section, key, utf8_wcstombs (value));
+}
+
+void
+StyleFile::set_string_array (String section, String key,
+                             std::vector<String> &value)
+{
+    StyleLines *lines = find_section (section);
+    if (lines) {
+        // find entry
+        StyleLines::iterator lit, last = lines->begin () + 1;
+        for (lit = last; lit != lines->end (); lit++) {
+            StyleLineType type = lit->get_type ();
+            if (type != SCIM_ANTHY_STYLE_LINE_SPACE)
+                last = lit;
+
+            String k;
+            lit->get_key (k);
+
+            // replace existing entry
+            if (k.length () > 0 && k == key) {
+                lit->set_value_array (value);
+                return;
+            }
         }
+
+        // append new entry if no mathced entry exists.
+        lines->insert (last + 1, StyleLine (this, key, value));
+
+    } else {
+        StyleLines &newsec = append_new_section (section);
+
+        // append new entry
+        newsec.push_back (StyleLine (this, key, value));
     }
+}
 
-    //
-    // append new section
-    //
-    m_sections.push_back (StyleLines ());
-    StyleLines &newsec = m_sections.back ();
-
-    // new section entry
-    String str = String ("[") + String (section) + String ("]");
-    newsec.push_back (StyleLine (this, str.c_str ()));
-
-    // new key entry
-    str = escape (key) + String ("=") + escape (value);
-    newsec.push_back (StyleLine (this, str.c_str ()));
+void
+StyleFile::set_string_array (String section, String key,
+                             std::vector<WideString> &value)
+{
+    std::vector<String> array;
+    std::vector<WideString>::iterator it;
+    for (it = value.begin (); it != value.end (); it++)
+        array.push_back (utf8_wcstombs (*it));
+    set_string_array (section, key, array);
 }
 
 bool
@@ -461,7 +577,6 @@ StyleFile::get_entry_list (StyleLines &lines, String section)
         if (it->size () <= 0)
             continue;
 
-        StyleLines::iterator lit;
         String s;
         (*it)[0].get_section (s);
         if (s == section) {
@@ -473,27 +588,40 @@ StyleFile::get_entry_list (StyleLines &lines, String section)
     return false;
 }
 
+bool
+StyleFile::get_key_list (std::vector<String> &keys, String section)
+{
+    StyleLines *lines = find_section (section);
+    if (!lines)
+        return false;
+
+    StyleLines::iterator lit;
+    for (lit = lines->begin (); lit != lines->end (); lit++) {
+        if (lit->get_type () != SCIM_ANTHY_STYLE_LINE_KEY)
+            continue;
+
+        String key;
+        lit->get_key (key);
+        keys.push_back (key);
+    }
+    return true;
+}
+
 void
 StyleFile::delete_key (String section, String key)
 {
-    StyleSections::iterator it;
-    for (it = m_sections.begin (); it != m_sections.end (); it++) {
-        if (it->size () <= 0)
-            continue;
+    StyleLines *lines = find_section (section);
+    if (!lines)
+        return;
 
-        String s, k;
-        (*it)[0].get_section (s);
-
-        if (s != section)
-            continue;
-
-        StyleLines::iterator lit;
-        for (lit = it->begin (); lit != it->end (); lit++) {
-            lit->get_key (k);
-            if (k == key) {
-                it->erase (lit);
-                return;
-            }
+    // find entry
+    StyleLines::iterator lit;
+    for (lit = lines->begin (); lit != lines->end (); lit++) {
+        String k;
+        lit->get_key (k);
+        if (k == key) {
+            lines->erase (lit);
+            return;
         }
     }
 }
@@ -516,6 +644,31 @@ StyleFile::delete_section (String section)
     }
 }
 
+Key2KanaTable *
+StyleFile::get_key2kana_table (String section)
+{
+    Key2KanaTable *table = NULL;
+
+    std::vector<String> keys;
+    bool success = get_key_list (keys, section);
+    if (success) {
+        table = new Key2KanaTable (utf8_mbstowcs (get_title ()));
+        std::vector<String>::iterator it;
+        for (it = keys.begin (); it != keys.end (); it++) {
+            std::vector<WideString> array;
+            get_string_array (array, section, *it);
+            String result, cont;
+            if (array.size () > 0)
+                result = utf8_wcstombs (array[0]);
+            if (array.size () > 1)
+                cont = utf8_wcstombs (array[1]);
+            table->append_rule (*it, result, cont);
+        }
+    }
+
+    return table;
+}
+
 void
 StyleFile::setup_default_entries (void)
 {
@@ -531,3 +684,48 @@ StyleFile::setup_default_entries (void)
     str = String ("Title") + String ("=") + escape (m_title);
     newsec.push_back (StyleLine (this, str.c_str ()));
 }
+
+StyleLines *
+StyleFile::find_section (const String  &section)
+{
+    // find section
+    StyleSections::iterator it;
+    for (it = m_sections.begin (); it != m_sections.end (); it++) {
+        if (it->size () <= 0)
+            continue;
+
+        String s;
+        (*it)[0].get_section (s);
+
+        if (s == section)
+            return &(*it);
+    }
+
+    return NULL;
+}
+
+StyleLines &
+StyleFile::append_new_section (const String &section)
+{
+    // append space before new section
+    if (!m_sections.empty()) {
+        StyleLines &sec = m_sections.back ();
+        if (sec.empty() ||
+            sec.back().get_type() != SCIM_ANTHY_STYLE_LINE_SPACE)
+        {
+            sec.push_back (StyleLine (this, ""));
+        }
+    }
+
+    //
+    // append new section
+    //
+    m_sections.push_back (StyleLines ());
+    StyleLines &newsec = m_sections.back ();
+
+    // new section entry
+    String str = String ("[") + String (section) + String ("]");
+    newsec.push_back (StyleLine (this, str.c_str ()));
+
+    return newsec;
+}
