@@ -24,6 +24,8 @@
 #include <scim.h>
 #include <scim_event.h>
 
+#include "scim_anthy_key2kana_table.h"
+
 using namespace scim;
 
 namespace scim_anthy {
@@ -48,17 +50,24 @@ class StyleLine
 {
 public:
     StyleLine (StyleFile  *style_file,
-               const char *line);
+               String      line);
+    StyleLine (StyleFile  *style_file,
+               String      key,
+               String      value);
+    StyleLine (StyleFile  *style_file,
+               String      key,
+               std::vector<String> &value);
     ~StyleLine ();
 
 public:
-    StyleLineType get_type    (void);
-    void          get_line    (String     &line) { line = m_line; }
-    bool          get_section (String     &section);
-    bool          get_key     (String     &key);
-    bool          get_value   (String     &value);
-    bool          get_value   (WideString &value);
-    void          set_value   (String      value);
+    StyleLineType get_type        (void);
+    void          get_line        (String     &line) { line = m_line; }
+    bool          get_section     (String     &section);
+    bool          get_key         (String     &key);
+    bool          get_value       (String     &value);
+    void          set_value       (String      value);
+    bool          get_value_array (std::vector<String> &value);
+    void          set_value_array (std::vector<String> &value);
 
 private:
     StyleFile     *m_style_file;
@@ -78,28 +87,54 @@ public:
 
     String get_encoding          (void);
     String get_title             (void);
+    String get_file_name         (void);
 
+    bool   get_section_list      (StyleSections &sections);
+    bool   get_entry_list        (StyleLines    &lines,
+                                  String         section);
+    bool   get_key_list          (std::vector<String> &keys,
+                                  String         section);
     bool   get_string            (String        &value,
                                   String         section,
                                   String         key);
     bool   get_string            (WideString    &value,
                                   String         section,
                                   String         key);
-    bool   get_section_list      (StyleSections &sections);
-    bool   get_entry_list        (StyleLines    &lines,
-                                  String         section);
+    bool   get_string_array      (std::vector<String> &value,
+                                  String         section,
+                                  String         key);
+    bool   get_string_array      (std::vector<WideString> &value,
+                                  String         section,
+                                  String         key);
 
     void   set_string            (String         section,
                                   String         key,
                                   String         value);
+    void   set_string            (String         section,
+                                  String         key,
+                                  WideString     value);
+    void   set_string_array      (String         section,
+                                  String         key,
+                                  std::vector<String> &value);
+    void   set_string_array      (String         section,
+                                  String         key,
+                                  std::vector<WideString> &value);
 
     void   delete_key            (String         section,
                                   String         key);
     void   delete_section        (String         section);
 
+public: // for getting specific data
+    Key2KanaTable *
+           get_key2kana_table    (String         section);
+
 private:
     void   clear                 (void);
     void   setup_default_entries (void);
+    StyleLines *
+           find_section          (const String  &section);
+    StyleLines &
+           append_new_section    (const String  &section);
 
 private:
     IConvert      m_iconv;
